@@ -8,9 +8,10 @@
 #' @param expr A numerical/logical expression which is the term to evaluate for the current preference. 
 #'       The objective is to search for minimal/maximal values of this expression (for \code{low}/\code{high}) or for 
 #'       logical \code{TRUE} values (for \code{true}).
+#'       For \code{low_}, \code{high_} and \code{true_}, the argument must be an expression, a call or a string.
 #' @param df (optional) A data frame, having the same structure (i.e., columns)
 #'        like that data frame, where this preference is evaluated later on. 
-#'        Causes a partial evaluation of the preference. Only the column names of \code{df} are relevant. 
+#'        Causes a partial evaluation of the preference and the preference is associated with this data frame.
 #'        See below for details.
 #' @param x An object to be tested if it is a base preference.
 #' 
@@ -24,7 +25,7 @@
 #'   \item{\code{low(a), high(a)}}{Search for minimal/maximal values of \code{a}, 
 #'         i.e., the induced order is the "smaller than" or "greater than" order on the values of \code{a}.
 #'         The values of \code{a} must be numeric values.}
-#'   \item{\code{true(a)}}{Searches for true values in logical expressions, i.e., \code{TRUE} is considered to be better than \code{FALSE}.
+#'   \item{\code{true(a)}}{Search for true values in logical expressions, i.e., \code{TRUE} is considered to be better than \code{FALSE}.
 #'         The values of \code{a} must be logical values.
 #'         For a tuplewise evaluation of a complex logical expression one has to use the \code{&} and \code{|} operators for logical AND/OR
 #'         (and not the \code{&&} and \code{||} operators).}
@@ -43,34 +44,16 @@
 #' The function \code{is.base_pref} returns \code{TRUE} if \code{x} is a preference object and \code{FALSE} otherwise.
 #' 
 #' 
-#' @section Partial Evaluation of Preferences:
-#' 
-#' If the optional parameter \code{df} is given, 
-#' then the expression is evaluated at the time of definition as far as possible.
-#' All variables occurring as columns in \code{df} remain untouched. For example, consider
-#' 
-#' \code{f <- function(x) 2*x} \cr
-#' \code{p <- true(cyl == f(1), mtcars)}
-#' 
-#' Then \code{p} is equivalent to the preference \code{true(cyl == 2)} as the variable \code{cyl} is a column in \code{mtcars}.
-#' The rows of \code{df} are not relevant, e.g., using \code{mtcars[0,]} instead of \code{mtcars} makes no difference.
-#' 
-#' The preference selection, i.e., \code{psel(mtcars, p)} can be invoked without the partial evaluation.
-#' But this results in an error, if the function \code{f} has meanwhile removed from the current environment.
-#' Hence it is safer to do an early partial evaluation of all preferences, as far as they contain user defined functions.
-#' 
-#' The partial evaluation can be done manually by \code{\link{eval.pref}}.
-#' 
-#' 
 #' @section Using Expressions in Preferences:
 #' 
 #' The \code{low_}, \code{high_} and \code{true_} preferences have the same functionality
 #' as \code{low}, \code{high} and \code{true} 
-#' but expect an expression \code{e} or symbol \code{e} as argument.
-#' For example, \code{low(a)} is equivalent to \code{low_(expression(a))} or \code{low_(as.symbol("a"))}. 
+#' but expect an expression, a call or a string as argument.
+#' For example, \code{low(a)} is equivalent to \code{low_(expression(a))} or \code{low_("a")}. 
+#' Lazy expressions (see the lazyeval package) are also possible.
 #' 
 #' 
-#' This is very helpful for developing your own base preferences. Assume you want to define a base Preference \code{false}
+#' This is helpful for developing your own base preferences. Assume you want to define a base Preference \code{false}
 #' as the dual of \code{true}. A definition like \code{false <- function(x) -true(x)} is the wrong approach, as 
 #' \code{psel(data.frame(a = c(1,2)), false(a == 1))} will result in the error "object 'a' not found".
 #' This is because \code{a} is considered as a variable and not as an (abstract) symbol to be evaluated later.
@@ -90,6 +73,27 @@
 #' the preference \code{low(df__[[1]])} is equivalent to \code{low(A)}.
 #' 
 #' 
+#' @section Partial Evaluation and Associated Data Frames:
+#' 
+#' If the optional parameter \code{df} is given, 
+#' then the expression is evaluated at the time of definition as far as possible.
+#' All variables occurring as columns in \code{df} remain untouched. For example, consider
+#' 
+#' \code{f <- function(x) 2*x} \cr
+#' \code{p <- true(cyl == f(1), mtcars)}
+#' 
+#' Then \code{p} is equivalent to the preference \code{true(cyl == 2)} as the variable \code{cyl} is a column in \code{mtcars}.
+#' Additionally the data set \code{mtcars} is associated with the preference \code{p}, 
+#' implying that the preference selection can be done with \code{\link{peval}}. 
+#' See \code{\link{set.df}} for details on associated data sets.
+#' 
+#' The preference selection, i.e., \code{psel(mtcars, p)} can be invoked without the partial evaluation.
+#' But this results in an error, if the function \code{f} has meanwhile removed from the current environment.
+#' Hence it is safer to do an early partial evaluation of all preferences, as far as they contain user defined functions.
+#' 
+#' The partial evaluation can be done manually by \code{\link{partial.eval.pref}}.
+#' 
+#' 
 #' @seealso See \code{\link{complex_pref}} how to compose complex preferences to retrieve e.g., the Skyline.
 #' See \code{\link{general_pref}} for functions applying to all kind of preferences.
 #' See \code{\link{base_pref_macros}} for more base preferences.
@@ -97,57 +101,71 @@
 #' @examples
 #' # define a preference with a score value combining mpg and hp
 #' p1 <- high(4 * mpg + hp)
-#' # Perform the preference selection
+#' # perform the preference selection
 #' psel(mtcars, p1)
 #' 
 #' # define a preference with a given function
 #' f <- function(x, y) (abs(x - mean(x))/max(x) + abs(y - mean(y))/max(y))
 #' p2 <- low(f(mpg, hp))
 #' psel(mtcars, p2)
+#' 
+#' # use partial evaluation for weighted scoring
+#' p3 <- high(mpg/sum(mtcars$mpg) + hp/sum(mtcars$hp), df = mtcars)
+#' p3
+#' # select Pareto optima
+#' peval(p3)
 NULL
 
-
 #' @rdname base_pref
 #' @export
 low <- function(expr, df = NULL) {
-  expr <- as.expression(substitute(expr))
-  return(eval.pref.internal(lowpref(expr, parent.frame()), df))
+  lowpref$new(get.lazy(substitute(expr), parent.frame()), df, substitute(df))
 }
 
 #' @rdname base_pref
 #' @export
 low_ <- function(expr, df = NULL) {
-  return(eval.pref.internal(lowpref(as.expression(expr), parent.frame()), df))
+  lowpref$new(get.lazy(expr, parent.frame()), df, substitute(df))
 }
 
 #' @rdname base_pref
 #' @export
 high <- function(expr, df = NULL) {
-  expr <- as.expression(substitute(expr))
-  return(eval.pref.internal(highpref(expr, parent.frame()), df))
+  highpref$new(get.lazy(substitute(expr), parent.frame()), df, substitute(df))
 }
 
 #' @rdname base_pref
 #' @export
 high_ <- function(expr, df = NULL) {
-  return(eval.pref.internal(highpref(as.expression(expr), parent.frame()), df))
+  highpref$new(get.lazy(expr, parent.frame()), df, substitute(df))
 }
 
 #' @rdname base_pref
 #' @export
 true <- function(expr, df = NULL) {
-  expr <- as.expression(substitute(expr))
-  return(eval.pref.internal(truepref(expr, parent.frame()), df))
+  truepref$new(get.lazy(substitute(expr), parent.frame()), df, substitute(df))
 }
 
 #' @rdname base_pref
 #' @export
 true_ <- function(expr, df = NULL) {
-  return(eval.pref.internal(truepref(as.expression(expr), parent.frame()), df))
+  truepref$new(get.lazy(expr, parent.frame()), df, substitute(df))
 }
 
 #' @rdname base_pref
 #' @export
 is.base_pref <- function(x) {
-  return(inherits(x, "basepref"))
+  inherits(x, "basepref")
 }
+
+
+# transform chracter/expression in lazy_eval
+#' @importFrom lazyeval as.lazy
+get.lazy <- function(expr_chr, env) {
+  # Unfold expression, if expression object given
+  if (is.expression(expr_chr)) expr_chr <- expr_chr[[1]]
+  # Create lazy_eval object
+  # Use given environment if expr_chr does not already have an associated environment
+  return(as.lazy(expr_chr, env))
+}
+
