@@ -17,7 +17,6 @@ gen_data <- function(N, cor, dim = 2) {
   names(data) <- paste0('x', 1:dim)
   return(data)
 }
- 
 
 
  
@@ -27,7 +26,7 @@ for (parallelity in c(FALSE, TRUE)) {
   options(rPref.parallel = parallelity)
 
   # Run all tests for pareto and intersection
-  for (`%op%` in list(`*`, `|`)) {
+  for (`%op%` in list(`*`, `|`)) { # `%op%` = `|`
   
     test_that("Compare BNL and Scalagon", {
       # 2-dim anticorrelated set with some outliers
@@ -46,26 +45,27 @@ for (parallelity in c(FALSE, TRUE)) {
       options(rPref.scalagon.alpha = 0)
       set1 <- sort(psel.indices(df2, low(x1) %op% low(x2)))
       set2 <- sort(psel.indices(df2g, low(x1) %op% low(x2)))
-      set3 <- sort(psel.indices(df3, low(x1) %op% low(x2) * low(x3)))
+      set3 <- sort(psel.indices(df3, low(x1) %op% (true(x2 < 0.5) * low(x3))))
       
       
       options(rPref.scalagon.alpha = 10)
       expect_equal(sort(psel.indices(df2, low(x1) %op% low(x2))), set1)
       expect_equal(sort(psel.indices(df2g, low(x1) %op% low(x2))), set2)
-      expect_equal(sort(psel.indices(df3, low(x1) %op% low(x2) * low(x3))), set3)
+      expect_equal(sort(psel.indices(df3, low(x1) %op% (true(x2 < 0.5) * low(x3)))), set3)
       
       # * Compare top-k preference selection
       
+      # Compare tuple indices AND level values
+      
       options(rPref.scalagon.alpha = 0)
-      set1 <- sort(psel.indices(df2, low(x1) %op% low(x2), at_least = 500))
-      set2 <- sort(psel.indices(df2g, low(x1) %op% low(x2), at_least = 20))
-      set3 <- sort(psel.indices(df3, low(x1) %op% low(x2) * low(x3), top_level = 2))
+      set1 <- arrange(psel.indices(df2, low(x1) %op% low(x2), at_least = 500, show_level = TRUE), .indices)
+      set2 <- arrange(psel.indices(df2g, high(x1) %op% low(x2), at_least = 20, show_level = TRUE), .indices)
+      set3 <- arrange(psel.indices(df3, high(x1) %op% (true(x2 < 0.5) * low(x3)), top_level = 2, show_level = TRUE), .indices)
       
       options(rPref.scalagon.alpha = 10)
-      expect_equal(sort(psel.indices(df2, low(x1) %op% low(x2), at_least = 500)), set1)
-      expect_equal(sort(psel.indices(df2g, low(x1) %op% low(x2), at_least = 20)), set2)
-      expect_equal(sort(psel.indices(df3, low(x1) %op% low(x2) %op% low(x3), top_level = 2)), set3)
-      
+      expect_equal(arrange(psel.indices(df2, low(x1) %op% low(x2), at_least = 500, show_level = TRUE), .indices), set1)
+      expect_equal(arrange(psel.indices(df2g, high(x1) %op% low(x2), at_least = 20, show_level = TRUE), .indices), set2)
+      expect_equal(arrange(psel.indices(df3, high(x1) %op% (true(x2 < 0.5) * low(x3)), top_level = 2, show_level = TRUE), .indices), set3)
       
     })
   }
